@@ -142,6 +142,11 @@ sub nth_twin_prime_approx {
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::nth_twin_prime_approx($n);
 }
+sub nth_ramanujan_prime {
+  my($n) = @_;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::nth_ramanujan_prime($n);
+}
 
 
 *is_prime          = \&Math::Prime::Util::PP::is_prime;
@@ -214,6 +219,12 @@ sub is_aks_prime {
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::is_aks_prime($n);
 }
+sub is_ramanujan_prime {
+  my($n) = @_;
+  return 0 if defined $n && int($n) < 0;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::is_ramanujan_prime($n);
+}
 sub is_mersenne_prime {
   my($p) = @_;
   _validate_positive_integer($p);
@@ -434,6 +445,11 @@ sub invmod {
   _validate_integer($n);
   return Math::Prime::Util::PP::invmod($a,$n);
 }
+sub sqrtint {
+  my($n) = @_;
+  _validate_integer($n);
+  return Math::Prime::Util::PP::sqrtint($n);
+}
 
 sub legendre_phi {
   my($x, $a) = @_;
@@ -574,6 +590,13 @@ sub vecreduce (&@) {    ## no critic qw(ProhibitSubroutinePrototypes)
   $count;
 }
 
+sub vecextract {
+  my($aref, $mask) = @_;
+  croak "vecextract first argument must be an array reference"
+    unless ref($aref) eq 'ARRAY';
+  return Math::Prime::Util::PP::vecextract(@_);
+}
+
 1;
 
 __END__
@@ -607,7 +630,7 @@ Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2014 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
+Copyright 2014-2015 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
