@@ -4,7 +4,7 @@ use warnings;
 
 BEGIN {
   $ntheory::AUTHORITY = 'cpan:DANAJ';
-  $ntheory::VERSION = '0.49';
+  $ntheory::VERSION = '0.50';
 }
 
 BEGIN {
@@ -45,6 +45,7 @@ See L<Math::Prime::Util> for complete documentation.
   verify_prime(cert)                  verify a primality certificate
   is_mersenne_prime(p)                is 2^p-1 prime or composite
   is_aks_prime(n)                     AKS deterministic test (slow)
+  is_ramanujan_prime(n)               is n a Ramanujan prime
 
 =head2 PROBABLE PRIME TESTS
 
@@ -64,6 +65,7 @@ See L<Math::Prime::Util> for complete documentation.
 
   primes([start,] end)                array ref of primes
   twin_primes([start,] end)           array ref of twin primes
+  ramanujan_primes([start,] end)      array ref of Ramanujan primes
   next_prime(n)                       next prime > n
   prev_prime(n)                       previous prime < n
   prime_count(n)                      count of primes <= n
@@ -80,6 +82,7 @@ See L<Math::Prime::Util> for complete documentation.
   twin_prime_count_approx(n)          fast approx count of twin primes
   nth_twin_prime(n)                   the nth twin prime (n=1 returns 3)
   nth_twin_prime_approx(n)            fast approximate nth twin prime
+  nth_ramanujan_prime(n)              the nth Ramanujan prime (Rn)
   legendre_phi(n,a)                   # below n not div by first a primes
   prime_precalc(n)                    precalculate primes to n
 
@@ -125,9 +128,11 @@ See L<Math::Prime::Util> for complete documentation.
   vecmin(@list)                       minimum of list of integers
   vecmax(@list)                       maximum of list of integers
   vecreduce { ... } @list             reduce / left fold applied to list
+  vecextract(\@list, mask)            select from list based on mask
   is_power(n)                         return k if n = p^k for integer p, max k
   is_power(n,k)                       return 1 if n = p^k for integer p and k
   is_power(n,k,\$root)                as above but set root to p.
+  sqrtint(n)                          integer square root
   gcd(@list)                          greatest common divisor
   lcm(@list)                          least common multiple
   gcdext(x,y)                         return (u,v,d) where u*x+v*y=d
@@ -160,6 +165,8 @@ See L<Math::Prime::Util> for complete documentation.
   lucas_sequence(n, P, Q, k)          (U_k,V_k,Q_k) for Lucas(P,Q) mod n
   bernfrac(n)                         Bernoulli number as (num,den)
   bernreal(n)                         Bernoulli number as BigFloat
+  harmfrac(n)                         Harmonic number as (num,den)
+  harmreal(n)                         Harmonic number as BigFloat
   stirling(n,m,[type])                Stirling numbers of 1st or 2nd type
 
 =head2 NON-INTEGER MATH
@@ -180,7 +187,7 @@ See L<Math::Prime::Util> for complete documentation.
 
 =head1 COPYRIGHT
 
-Copyright 2011-2014 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
+Copyright 2011-2015 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
